<?php

/**
 * @file
 * Administrative callbacks for PATH BREADCRUMBS UI module.
 */

/**
 * Page callback for module settings page.
 */
function eavfield_ui_attributes_list($form, &$form_state) {
  $result = eavfield_attributes_load_multiple();
  $form_state['storage']['objects'] = $result;
  $form['#tree'] = TRUE;

  foreach ($result as $attr) {
    $elem_name = 'attr_' . $attr->aid;
    $form[$elem_name]['title']['#markup'] = $attr->name . '</sup>';
    $form[$elem_name]['name']['#markup'] = $attr->machine_name;
    $form[$elem_name]['group_name']['#markup'] = !empty($attr->group_name) ? $attr->group_name : '<all>';
    $form[$elem_name]['type']['#markup'] = $attr->type;
    $form[$elem_name]['#parents'] = array('attr', $attr->aid);

    $operations = array();

    if ($attr->disabled == TRUE) {
      $operations[] = array(
        'title' => t('Enable'),
        'href' => 'admin/structure/eavfield_status/' . $attr->aid . '/enable/' . drupal_get_token('eav'),
      );
    }

    $operations[] = array(
      'title' => t('Edit'),
      'href' => 'admin/structure/eavfield/' . $attr->aid . '/edit',
    );

    if ($attr->disabled == FALSE) {
      $operations[] = array(
        'title' => t('Disable'),
        'href' => 'admin/structure/eavfield_status/' . $attr->aid . '/disable/' . drupal_get_token('eav'),
      );
    }

    $form[$elem_name]['actions'] = array(
      '#theme' => 'links__ctools_dropbutton',
      '#links' => $operations,
      '#attributes' => array('class' => array('links', 'inline')),
    );

    $form[$elem_name]['weight'] = array(
      '#type' => 'weight',
      '#default_value' => $attr->weight,
      '#delta' => 100,
      '#attributes' => array('class' => array('eavfield-ui-table-weight')),
    );

    $form[$elem_name]['disabled'] = array(
      '#type' => 'value',
      '#value' => $attr->disabled,
    );
  }

  $form['actions'] = array(
    '#type' => 'action',
  );

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  $form['#attached']['css'][] = drupal_get_path('module', 'eavfield_ui') . '/css/eavfield_ui.css';
  return $form;
}


/**
 * Submit callback for path_breadcrumbs_ui_breadcrumbs_list form.
 */
function eavfield_ui_attributes_list_submit($form, &$form_state) {
  if (!empty($form_state['values']['attr'])) {
    foreach ($form_state['values']['attr'] as $aid => $value) {
      if (isset($value['weight'])) {
        $attr = $form_state['storage']['objects'][$aid];
        $attr->weight = $value['weight'];
        eavfield_attributes_save($attr);
      }
    }
  }

  drupal_set_message(t('Attribute was updated.'));
}


/**
 * Page callback for module settings page.
 */
function eavfield_ui_edit_form($form, &$form_state) {
  $form = array(
    '#tree' => TRUE,
  );

  if (isset($form_state['build_info']['args'][0]->aid)) {
    $attr = $form_state['build_info']['args'][0];
  }
  else {
    $attr = FALSE;
  }

  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#description' => t('Attribute name'),
    '#max_length' => 50,
    '#default_value' => isset($attr->name) ? $attr->name : '',
    '#required' => TRUE,
  );

  $form['attr']['aid'] = array(
    '#type' => 'value',
    '#value' => $attr ? $attr->aid : NULL,
  );

  $form['attr']['machine_name'] = array(
    '#type' => 'machine_name',
    '#default_value' => isset($attr->machine_name) ? $attr->machine_name : '',
    '#maxlength' => 32,
    '#machine_name' => array(
//      'source'=>array(),
      'exists' => '_eavfield_machine_name_exists',
    ),
  );

  $form['attr']['group_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Group'),
    '#description' => t('Attribute group machine_name'),
    '#default_value' => isset($attr->group_name) ? $attr->group_name : '',
    '#max_length' => 32,
  );

  $form['attr']['type'] = array(
    '#type' => 'select',
    '#title' => t('Type'),
    '#description' => t('Type attribute'),
    '#default_value' => isset($attr->type) ? $attr->type : 'int',
    '#max_length' => 10,
    '#required' => TRUE,
    '#options' => _eavfield_attr_getType(),
  );

  $form['attr']['disabled'] = array(
    '#type' => 'checkbox',
    '#title' => t('Disabled'),
    '#default_value' => isset($attr->disabled) ? $attr->disabled : 0,
  );

  $form['attr']['weight'] = array(
    '#type' => 'weight',
    '#title' => t('Weight'),
    '#default_value' => isset($attr->weight) ? $attr->weight : 0,
    '#delta' => 10,
  );

  $form['items'] = array(
    '#type' => 'container',
    '#states' => array(
      'visible' => array(
        ':input[name="attr[type]"]' => array('value' => 'list'),
      ),
    ),
  );

  if (isset($attr->itemlist) && count($attr->itemlist)) {
    $count = count($attr->itemlist);
    $form['items']['old'] = array(
      '#type' => 'fieldset',
      '#title' => t('Old items: !count', array('!count' => $count)),
      '#collapsible' => TRUE,
      '#collapsed' => $count > 10 ? TRUE : FALSE,
    );

    foreach ($attr->itemlist as $lid => $item_name) {
      $form['items']['old'][$lid]['lid'] = array(
        '#type' => 'value',
        '#value' => $lid,
      );

      $form['items']['old'][$lid]['name'] = array(
        '#type' => 'textfield',
        '#default_value' => $item_name,
        '#max_length' => 255,
        '#states' => array(
          'disabled' => array(
            ':input[name="items[old][' . $lid . '][delete]"]' => array('checked' => TRUE),
          ),
        ),
      );

      $form['items']['old'][$lid]['delete'] = array(
        '#type' => 'checkbox',
        '#title' => t('Delete'),
        '#default_value' => 0,
      );
    }
  }

  $form['items']['new'] = array(
    '#type' => 'textarea',
    '#title' => t('New items'),
    '#default_value' => '',
  );

  $form['actions'] = array(
    '#type' => 'action',
  );

  $form['actions']['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#name' => 'save',
  );
  if (isset($attr->aid)) {
    $form['actions']['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
      '#name' => 'delete',
    );
  }
  return $form;
}

function eavfield_ui_edit_form_validate($form, &$form_state) {
  if (isset($form_state['values']['attr']['group_name']) && empty($form_state['values']['attr']['group_name'])) {
    $form_state['values']['attr']['group_name'] = NULL;
  }

  $items_old = array();
  $items_new = array();

  if (isset($form_state['values']['attr']['type']) && $form_state['values']['attr']['type'] != 'list') {
    unset($form_state['values']['items']);
  }
  else {
    if (isset($form_state['values']['items']['old'])) {
      foreach ($form_state['values']['items']['old'] as $item) {
        if (!$item['delete'] && !empty($item['name'])) {
          $items_old[$item['lid']] = $item['name'];
        }
      }
    }

    if (isset($form_state['values']['items']['new']) && !empty($form_state['values']['items']['new'])) {
      //парсим новые значения для атрибута
      $list = explode("\n", $form_state['values']['items']['new']);
      $list = array_map('trim', $list);
      $list = array_filter($list, 'strlen');
      foreach ($list as $new_name) {
        $items_new[$new_name] = $new_name;
      }
      $items_new = array_values($items_new);
    }
  }

  $form_state['values']['attr']['itemlist'] = $items_old;
  $form_state['values']['attr']['itemlist_new'] = $items_new;
}

function eavfield_ui_edit_form_submit($form, &$form_state) {
  if (!isset($form_state['values']['attr'])) {
    return;
  }

  $attr = (object) $form_state['values']['attr'];
  if (isset($form_state['values']['name'])) {
    $attr->name = $form_state['values']['name'];
  }
  else {
    $attr->name = $attr->machine_name;
  }

  if ($form_state['triggering_element']['#name'] == 'save') {
    if (isset($attr)) {
      eavfield_attributes_save($attr);
    }
  }

  if ($form_state['triggering_element']['#name'] == 'delete') {
    //Удаляем атрибут
    eavfield_attributes_delete($attr);
    drupal_goto('admin/structure/eavfield/list');
  }
}


