<?php
/**
 * Groups list
 */
function eavfield_group_list($all = TRUE) {
  $items = array();

  if ($all) {
    $items['all'] = 'all';
  }
  $result = db_select('eavfield_attr', 'n')
    ->fields('n', array('group_name'))
    ->groupBy('n.group_name')
    ->condition('n.disabled', 0)
    ->orderBy('n.weight', 'ASC')
    ->execute();

  foreach ($result as $item) {
    if (!empty($item->group_name)) {
      $items[$item->group_name] = $item->group_name;
    }
  }

  return $items;
}

/**
 * Attributes list
 */
function eavfield_attributes_list($group_name = 'all') {
  $query = db_select('eavfield_attr', 'n')
    ->fields('n', array('aid', 'name'))
    ->orderBy('n.weight', 'ASC')
    ->condition('n.disabled', 0);

  if ($group_name != 'all') {
    $query = $query->condition('n.group_name', $group_name);
  }

  $attributes = $query->execute()->fetchAllKeyed();
  return $attributes;
}

/**
 * Attribute load
 * @param $aid int
 * @return mixed
 */
function eavfield_attributes_load($aid) {
  $attr = db_select('eavfield_attr', 'n')
    ->fields('n')
    ->condition('n.aid', $aid)
    ->execute()
    ->fetchObject();

  if ($attr) {
    $attr->itemlist = eavfield_itemlist_load_options($attr->aid);
  }

  return $attr;
}

function eavfield_attributes_load_multiple($aids = NULL, $params = array()) {
  $query = db_select('eavfield_attr', 'n')
    ->fields('n')
    ->orderBy('n.weight', 'ASC');

  if (!is_null($aids) && count($aids)) {
    $query = $query->condition('n.aid', $aids, 'IN');
  }

  if (count($params)) {
    foreach ($params as $key => $value) {
      $query = $query->condition('n.' . $key, $value);
    }
  }
  $attributes = $query->execute()->fetchAllAssoc('aid');
  if (count($attributes)) {
    foreach ($attributes as $aid => $attr) {
      $attributes[$aid]->itemlist = eavfield_itemlist_load_options($attr->aid);
    }
  }
  return $attributes;
}

function eavfield_attributes_save($attr) {
  $values = array(
    'machine_name' => isset($attr->machine_name) ? $attr->machine_name : time(),
    'name' => isset($attr->name) ? $attr->name : 'Attribute',
    'group_name' => isset($attr->group_name) && !empty($attr->machine_name) ? $attr->group_name : NULL,
    'type' => isset($attr->type) ? $attr->type : 'int',
    'disabled' => isset($attr->disabled) ? $attr->disabled : 0,
    'weight' => isset($attr->weight) ? $attr->weight : 0,
  );

  if (isset($attr->aid) && is_numeric($attr->aid)) {
    //Обновляем атрибут
    db_update('eavfield_attr')
      ->fields($values)
      ->condition('aid', $attr->aid)
      ->execute();

    drupal_set_message(t('EAV attribute !name edit', array('!name' => $attr->name)));
  }
  else {
    //Создаем новый атрибут
    $aid = db_insert('eavfield_attr')
      ->fields($values)
      ->execute();

    if ($aid) {
      $attr->aid = $aid;
    }
    drupal_set_message(t('EAV attribute !name create', array('!name' => $attr->name)));
  }

  if ($attr && $attr->aid) {
    if ($attr->type != 'list') {
      db_delete('eavfield_list')
        ->condition('aid', $attr->aid)
        ->execute();
    }
    else {
      //Обработка списка значений
      //Список старых значений
      $itemlist_old = eavfield_itemlist_load_options($attr->aid);
      //Список измененных значений
      $itemlist = isset($attr->itemlist) ? $attr->itemlist : array();
      //Список новых значений
      $itemlist_new = isset($attr->itemlist_new) ? $attr->itemlist_new : array();
      _eavfield_itemlist_update($attr->aid, $itemlist_old, $itemlist, $itemlist_new);
    }

  }
  return $attr;
}

/**
 * Удаляем атрибут
 */
function eavfield_attributes_delete($attr) {
  if ($attr && $attr->aid) {
    db_delete('eavfield_list')
      ->condition('aid', $attr->aid)
      ->execute();
    db_delete('eavfield_attr')
      ->condition('aid', $attr->aid)
      ->execute();
    drupal_set_message(t('EAV attribute !name delete', array('!name' => $attr->name)));
  }
}


/**
 * Helper function
 * ОБработка списка значений для list
 */
function _eavfield_itemlist_update($aid, $itemlist_old, $itemlist, $itemlist_new) {

  foreach ($itemlist_new as $num => $item) {
    $key = array_search($item, $itemlist_old);
    if ($key !== FALSE) {
      //Новое значение уже присутствует в текущем списке
      //его создавать нет необходимости,
      //просто добавим его в список "после обновления"
      $itemlist[$key] = $item;
      unset($itemlist_new[$num]);
    }
  }

  foreach ($itemlist as $lid => $item) {
    $key = array_search($item, $itemlist_new);
    if ($key !== FALSE) {
      //Обновленное значение есть в всписке новых
      //его создавать нет необходимости
      unset($itemlist_new[$key]);
    }
    if ($itemlist_old[$lid] != $item) {
      db_update('eavfield_list')
        ->fields(array(
          'aid' => $aid,
          'name' => $item,
        ))
        ->condition('lid', $lid)
        ->execute();
    }
    unset($itemlist_old[$lid]);
  }

  //Создаем новые значения
  foreach ($itemlist_new as $num => $item) {
    $id = db_insert('eavfield_list')
      ->fields(array(
        'aid' => $aid,
        'name' => $item,
      ))
      ->execute();
  }

  //Удаляем лишние значения
  if (count($itemlist_old)) {
    db_delete('eavfield_list')
      ->condition('aid', $aid)
      ->condition('lid', array_keys($itemlist_old), 'IN')
      ->execute();

    //TODO: Предусмотреть возможность удаления значений из полей????
  }
}


/**
 * @param $lid
 * @param $aid
 */
function eavfield_itemlist_load($lid, $aid = NULL) {

  $query = db_select('eavfield_list', 'n')
    ->fields('n')
    ->condition('n.lid', $lid);
  if (!is_null($aid) && is_numeric($aid)) {
    $query = $query->condition('n.aid', $aid);
  }
  $itemlist = $query->execute()->fetchObject();
  return $itemlist;
}

/**
 * Список значений для определенного аттрибута
 */
function eavfield_itemlist_load_options($aid = NULL) {
  $items = array();

  $query = db_select('eavfield_list', 'n')
    ->fields('n', array('lid', 'name'))
    ->orderBy('n.name', 'ASC');
  if (!is_null($aid) && is_numeric($aid)) {
    $query = $query->condition('n.aid', $aid);
  }
  $items = $query->execute()->fetchAllKeyed();;
  return $items;
}

/**
 * Возвращает значение в зависимоти от типа атрибута
 *
 */
function eavfield_value_to_attr_return($value, $attr) {
  $return_value = FALSE;
  if ($attr->type == 'int') {
    if (is_numeric($value)) {
      $return_value = $value;
    }
  }

  if ($attr->type == 'list') {
    $itemlist = eavfield_itemlist_load($value, $attr->aid);
    if ($itemlist) {
      $return_value = check_plain($itemlist->name);
    }
  }

  if ($attr->type == 'bool') {
    if (is_numeric($value) && $value > 0) {
      $return_value = 1;
    }
    else {
      $return_value = 0;
    }
  }
  return $return_value;
}

/**
 * Проверяет, существует ли атрибут с заданным machine_name
 */
function _eavfield_machine_name_exists($value) {
  $aid = db_select('eavfield_attr', 'n')
    ->fields('n')
    ->condition('n.machine_name', $value)
    ->execute()
    ->fetchObject();

  return $aid;
}

function _eavfield_attr_getType() {
  $types = array(
    'int' => t('Integer'),
    'bool' => t('Boolean'),
    'list' => t('List'),
  );
  return $types;
}
